/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

/**
 * SECTION:kiba-drawable-object
 * @short_description: Base class for drawable objects on #KibaDock.
 * @see_also: #KibaWin,#KibaDock,#KibaObject
 *
 * #KibaObject implementation that allows custom drawing.
 */

#ifndef __KIBA_DRAWABLE_OBJECT_H__
#define __KIBA_DRAWABLE_OBJECT_H__

#include <gtk/gtk.h>
#include <glib.h>
#include <cairo.h>

#include <kiba-object.h>
#include <kiba-cairo-utils.h>

G_BEGIN_DECLS

typedef struct _KibaDrawableObject KibaDrawableObject;
typedef struct _KibaDrawableObjectClass KibaDrawableObjectClass;

/**
 * KibaObjectCustomAlphaFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The alpha return.
 * @Param4: User Data.
 *
 * Method used for custom alpha animations.
 */
typedef void (*KibaDrawableObjectAlphaFunc) (KibaDrawableObject*, gfloat*, gpointer);

#define KIBA_TYPE_DRAWABLE_OBJECT           (kiba_drawable_object_get_type ())
#define KIBA_DRAWABLE_OBJECT(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_DRAWABLE_OBJECT, KibaDrawableObject))
#define KIBA_DRAWABLE_OBJECT_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_TYPE_DRAWABLE_OBJECT,  KibaDrawableObjectClass))
#define KIBA_IS_DRAWABLE_OBJECT(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_DRAWABLE_OBJECT))
#define KIBA_IS_DRAWABLE_OBJECT_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_DRAWABLE_OBJECT))
#define KIBA_DRAWABLE_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_DRAWABLE_OBJECT, KibaDrawableObjectClass))

/**
 * KibaObjectAlphaData:
 *
 * Type for alpha animations.
 */
typedef struct {
	KibaDrawableObjectAlphaFunc alpha_func;
	gpointer data;
}KibaDrawableObjectAlphaData;

typedef void (* RenderObjectFunc) (KibaDrawableObject*, cairo_t*);

struct _KibaDrawableObject {
	KibaObject parent;
	
	CairoSurfaceData *rendered_object; /**< Object surface with morph and ontop features */
	
	// offset due to visual features
	gfloat rendered_x;
	gfloat rendered_y;
	
	GList *alpha_data; /**< list of all KibaDrawableObjectAlphaData's */
	gfloat alpha; /**< current alpha value */
	
	/*
	 * Method updates the object surface buffer, if a buffer is used.
	 * Can be overwritten by KibaObject implementations.
	 */
	void (* update_surface) (KibaDrawableObject*);
	/*
	 * Returns a reference on the object surface.
	 */
	cairo_surface_t* (* get_surface) (KibaDrawableObject*);
	/*
	 * Method renders the object to a drawing context.
	 * Can be overwritten by KibaObject implementations.
	 * Default is to render the surface buffer to the context,
	 * if it is not NULL.
	 */
	RenderObjectFunc render_object;
};

struct _KibaDrawableObjectClass {
	KibaObjectClass parent_class;

	// signals
	void (* switch_event) (KibaDrawableObject *);
	void (* alpha_changed) (KibaDrawableObject*);
	void (* area_changed) (KibaDrawableObject*);
	void (* queued_draw) (KibaDrawableObject*);
};

GtkWidget* kiba_drawable_object_new (const gchar *object_id,
	gfloat scale_x, gfloat scale_y);

void kiba_drawable_object_update_area (KibaDrawableObject *object);

GdkRectangle kiba_drawable_object_get_area (KibaDrawableObject *object);

void kiba_drawable_object_switch_surface (KibaDrawableObject *object);


void kiba_drawable_object_queue_draw (KibaDrawableObject *object);


void kiba_drawable_object_set_alpha (KibaDrawableObject *object,
                                     gfloat alpha);

gfloat kiba_drawable_object_get_alpha (KibaDrawableObject *object);

void kiba_drawable_object_update_alpha (KibaDrawableObject *object);

void kiba_drawable_object_remove_alpha_function_by_data (KibaDrawableObject *object,
					        gpointer    user_data);

void kiba_drawable_object_add_alpha_function (KibaDrawableObject *object,
                                     KibaDrawableObjectAlphaFunc alpha_func,
                                     gpointer user_data);

void kiba_drawable_object_queue_update_surface(KibaDrawableObject *drawable_object);

void kiba_drawable_object_queue_render_object(KibaDrawableObject *drawable_object);

void kiba_drawable_object_update_surface(KibaDrawableObject *drawable_object,
		KibaWin *win);

GdkWindow* kiba_drawable_object_event_window(KibaDrawableObject *object);

void
kiba_drawable_object_set_render_object_method(
		KibaDrawableObject *drawable_object,
		RenderObjectFunc render_object);

RenderObjectFunc
kiba_drawable_object_get_render_object_method(
		KibaDrawableObject *drawable_object);

G_END_DECLS

#endif
