/*
 *  Copyright (C) 2006-2007 Daniel Bessler <danielb@kiba-dock.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA.
 *
 *  Author : Daniel Bessler <danielb@kiba-dock.org>
*/

/**
 * SECTION:kiba-object
 * @short_description: Base class for objects on #KibaDock.
 * @see_also: #KibaWin,#KibaDock
 *
 * Abstract base class for objects on #KibaDock.
 * #KibaObject is responsible for scaling/positiong objects on #KibaDock.
 */

#ifndef __KIBA_OBJECT_H__
#define __KIBA_OBJECT_H__

#include <gtk/gtk.h>
#include <glib.h>

#include <kiba-window.h>
#include <kiba-types.h>

G_BEGIN_DECLS

#define KIBA_TYPE_OBJECT           (kiba_object_get_type ())
#define KIBA_OBJECT(obj)           (G_TYPE_CHECK_INSTANCE_CAST ((obj), KIBA_TYPE_OBJECT, KibaObject))
#define KIBA_OBJECT_CLASS(obj)     (G_TYPE_CHECK_CLASS_CAST ((obj), KIBA_TYPE_OBJECT,  KibaObjectClass))
#define KIBA_IS_OBJECT(obj)        (G_TYPE_CHECK_INSTANCE_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_IS_OBJECT_CLASS(obj)  (G_TYPE_CHECK_CLASS_TYPE ((obj), KIBA_TYPE_OBJECT))
#define KIBA_OBJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), KIBA_TYPE_OBJECT, KibaObjectClass))

typedef struct _KibaObject KibaObject;
typedef struct _KibaObjectClass KibaObjectClass;
typedef struct _KibaObjectProgress KibaObjectProgress;

#include <gtk/gtk.h>
#include <glib.h>
#include <cairo.h>

/**
 * KibaObjectCustomScaleFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The width scale factor return.
 * @Param3: The height scale factor return.
 * @Param4: User Data.
 *
 * Method used for custom scaling.
 */
typedef void (*KibaObjectScaleFunc) (KibaObject*, gfloat*, gfloat*, gpointer);

/**
 * KibaObjectCustomMaxScaleFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The maximal width scale factor return.
 * @Param3: The maximal height scale factor return.
 * @Param4: User Data.
 *
 * Gets maximal scale factors for a scaling method.
 */
typedef void (*KibaObjectMaxScaleFunc) (KibaObject*, gfloat*, gfloat*, gpointer);

/**
 * KibaObjectCustomOffsetFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: The width offset return.
 * @Param3: The height offset return.
 * @Param4: User Data.
 *
 * Method used for custom offset on the dock.
 */
typedef void (*KibaObjectOffsetFunc) (KibaObject*, gfloat*, gfloat*, gpointer);

/**
 * KibaObjectCustomMaxOffsetFunc:
 * @Param1: The KibaObject Instance.
 * @Param2: Offset to the left.
 * @Param3: Offset to the right.
 * @Param4: Offset to the top.
 * @Param5: Offset to the bottom.
 * @Param6: User Data.
 *
 * Method used for custom offset on the dock.
 */
typedef void (*KibaObjectMaxOffsetFunc) (KibaObject*,
	gfloat*, gfloat*, gfloat*, gfloat*, gpointer);

typedef enum {
	KIBA_OBJECT_ON_DOCK = 0,
	KIBA_OBJECT_LEFT_OF_DOCK,
	KIBA_OBJECT_RIGHT_OF_DOCK
}KibaObjectPosition;

/**
 * KibaObjectScaleData:
 * @scale_func: method that calculates the current scale factor.
 * @max_func: method that returns maximal scale factor.
 * @scale_widget: should the widget be scaled, or only the surface?
 * @data: user data
 *
 * Type for scale animations.
 */
typedef struct {
	KibaObjectScaleFunc scale_func;
	KibaObjectMaxScaleFunc max_func;
	gboolean scale_widget;
	gpointer data;
}KibaObjectScaleData;

/**
 * KibaObjectOffsetData:
 * @place_func: method that calculates the current offset.
 * @max_func: method that returns maximal offset.
 * @translate_widget: should the widget be translated, or only the surface?
 * @data: user data
 *
 * Type for surface offset animations.
 */
typedef struct {
	KibaObjectOffsetFunc place_func;
	KibaObjectMaxOffsetFunc max_func;
	gboolean translate_widget;
	gpointer data;
}KibaObjectOffsetData;

/**
 * KibaObjectState:
 * @KIBA_OBJECT_DRAGGING: dnd active?
 * @KIBA_OBJECT_PRELIGHTED: object prelighted?
 * @KIBA_OBJECT_URGENT: object urgent?
 * @KIBA_OBJECT_ACTIVE: object active?
 * @KIBA_OBJECT_WORKING: object working?
 * @KIBA_OBJECT_GREYED: object greyed?
 *
 * Object flags.
 */
typedef enum {
	KIBA_OBJECT_DRAGGING = 1,
	KIBA_OBJECT_PRELIGHTED,
	KIBA_OBJECT_URGENT,
	KIBA_OBJECT_ACTIVE,
	KIBA_OBJECT_WORKING,
	KIBA_OBJECT_GREYED
}KibaObjectState;

struct _KibaObjectProgress {
	guint id;
	gint value;
};

struct _KibaObject {
	GtkBin bin;

	KibaObjectPosition position;
	gint state; /**<< state flags **/

	KibaAllocation alloc;
	
	GtkRequisition size;

	gfloat scale_x; /**< scale factor (surface*widget) */
	gfloat scale_y; /**< scale factor (surface*widget) */
	gfloat surface_scale_x; /**< surface scale factor */
	gfloat surface_scale_y; /**< surface scale factor */
	gfloat widget_scale_x; /**< surface scale factor */
	gfloat widget_scale_y; /**< surface scale factor */
	
	gfloat surface_offset_x;
	gfloat surface_offset_y;
	gfloat widget_offset_x;
	gfloat widget_offset_y;
	
	gfloat destroy_factor; /**< Scale factor for the destroy animation */
	
	gfloat cfg_scale_x; /**< widget width scale factor */
	gfloat cfg_scale_y; /**< widget height scale factor */

	GList *offset_data;/**< list of all KibaObjectOffsetData's */
	GList *scale_data; /**< list of all KibaObjectScaleData's */
	
	GList *features_morph_front;
	GList *features_morph_back;
	GList *features_ontop_front;
	GList *features_ontop_back;
	GList *features_ontop_dock_front;
	GList *features_ontop_dock_back;
	GList *features_gimmick_front;
	GList *features_gimmick_back;
};

struct _KibaObjectClass {
	GtkBinClass parent_class;

	/* signals */
	void (* urgent_toggled) (KibaObject  *object,
                             gboolean urgent);
	void (* active_toggled) (KibaObject  *object,
                             gboolean active);
	void (* working_toggled) (KibaObject  *object,
                              gboolean working);
	void (* process_toggled) (KibaObject  *object,
			gint id,
			gboolean started);
	void (* process_updated) (KibaObject  *object,
			gint id,
			gint value);
	void (* dragging_toggled) (KibaObject  *object,
                              gboolean dragging);
	void (* prelight_toggled) (KibaObject  *object,
                              gboolean prelight);
	void (* greyed_toggled) (KibaObject  *object,
                              gboolean greyed);
	void ( *scale_changed) (KibaObject  *object,
			    gboolean widget_scale_changed,
			    gboolean surface_scale_changed);
	void (* offset_changed) (KibaObject  *object,
			    gboolean widget_offset_changed,
			    gboolean surface_offset_changed);
};

#include <kiba-visual-feature.h>

void kiba_object_set_icon_geometry (KibaObject *object,
                                    KibaWindow *window);

int kiba_object_launch (KibaObject *object,
                        const gchar *command);

gboolean kiba_object_get_state (KibaObject *object,
                                KibaObjectState state);

void kiba_object_set_state (KibaObject *object,
                            KibaObjectState state);

void kiba_object_unset_state (KibaObject *object,
                              KibaObjectState state);

gboolean kiba_object_is_on_dock (KibaObject *object);

void kiba_object_update_scale_factor (KibaObject *object);


void kiba_object_get_scale(KibaObject *object,
		gfloat *scale_x, gfloat *scale_y);

void kiba_object_get_surface_scale(KibaObject *object,
		gfloat *surface_scale_x, gfloat *surface_scale_y);

void kiba_object_get_widget_scale(KibaObject *object,
		gfloat *widget_scale_x, gfloat *widget_scale_y);

void kiba_object_add_scale_function (KibaObject *object,
		KibaObjectScaleFunc scale_func,
		KibaObjectMaxScaleFunc max_func,
		gboolean scale_widget,
		gpointer user_data);

void kiba_object_remove_scale_function_by_data (KibaObject *object,
                                                gpointer user_data);


void kiba_object_get_surface_offset(KibaObject *object,
		gfloat *offset_x, gfloat *offset_y);

void kiba_object_get_widget_offset(KibaObject *object,
		gfloat *offset_x, gfloat *offset_y);

void kiba_object_update_offset (KibaObject *object);

void kiba_object_add_offset_function (KibaObject *object,
		KibaObjectOffsetFunc place_func,
		KibaObjectMaxOffsetFunc max_func,
		gboolean translate_widget,
		gpointer user_data);

void kiba_object_remove_offset_function_by_data (KibaObject *object,
                                                 gpointer user_data);


void kiba_object_set_tooltip_text(KibaObject *object,
                                  const gchar *text);

void kiba_object_set_title (KibaObject *object,
                            const gchar *text);

const gchar* kiba_object_get_title (KibaObject *object);

void kiba_object_show_title (KibaObject *object);

void kiba_object_hide_title (KibaObject *object);

gboolean kiba_object_get_destroying (KibaObject *object);

void kiba_object_destroy (KibaObject *object);

float kiba_object_get_destroy_factor(KibaObject *object);

void kiba_object_destroy_factor_func(KibaObject* object,
	gfloat* x, gfloat* y, gpointer data);

void kiba_object_make_sensitive(KibaObject *object,
	gboolean sensitive);


void kiba_object_enable_feature(KibaObject *object,
			KibaVisualFeatureObject *feature_object);

void kiba_object_disable_feature(KibaObject *object,
			KibaVisualFeatureObject *feature_object);

GList* kiba_object_get_processes(KibaObject *object);

KibaObjectProgress* kiba_object_start_progress(KibaObject *object);

void kiba_object_stop_progress(KibaObject *object,
		KibaObjectProgress *progress);

void kiba_object_update_progress(KibaObject *object,
		KibaObjectProgress *progress, gint value);

G_END_DECLS

#endif
